@@ -1,6 +1,6 @@
 ;;; run-tests.body.scm --- Bytestructures test suite.
 
-;; Copyright © 2015 Taylan Ulrich Bayırlı/Kammer <taylanbayirli@gmail.com>
+;; Copyright © 2015, 2021 Taylan Kammer <taylan.kammer@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -171,7 +171,52 @@
          (bytevector-u16-native-set! bv 1 321))
        (test-eqv "ref" 321 (getter bv y))
        (test-eqv "set" 456 (begin (setter bv y 456)
-                                  (getter bv y)))))))
+                                  (getter bv y))))))
+
+  (test-group "anonymous-union"
+    (test-assert "create"
+      (bs:struct
+       `((x ,uint8)
+         (union
+          ((a ,uint16)
+           (b ,uint32))))))
+    (test-group "aligned"
+      (define bs
+        (bytestructure
+         (bs:struct
+          `((union
+             ((x ,uint8)
+              (y ,uint16)))
+            (union
+             ((a ,uint32)
+              (b ,uint64)))))))
+      (bytevector-u32-native-set! (bytestructure-bytevector bs) 8 321)
+      (test-eqv "ref1" 321 (bytestructure-ref bs 'a))
+      (bytevector-u64-native-set! (bytestructure-bytevector bs) 8 456)
+      (test-eqv "ref2" 456 (bytestructure-ref bs 'b))
+      (test-eqv "set1" 789 (begin (bytestructure-set! bs 'a 789)
+                                  (bytestructure-ref bs 'a)))
+      (test-eqv "set2" 987 (begin (bytestructure-set! bs 'b 987)
+                                  (bytestructure-ref bs 'b))))
+    (test-group "packed"
+      (define bs
+        (bytestructure
+         (bs:struct
+          #t
+          `((union
+             ((x ,uint8)
+              (y ,uint16)))
+            (union
+             ((a ,uint32)
+              (b ,uint64)))))))
+      (bytevector-u32-native-set! (bytestructure-bytevector bs) 2 321)
+      (test-eqv "ref1" 321 (bytestructure-ref bs 'a))
+      (bytevector-u64-native-set! (bytestructure-bytevector bs) 2 456)
+      (test-eqv "ref2" 456 (bytestructure-ref bs 'b))
+      (test-eqv "set1" 789 (begin (bytestructure-set! bs 'a 789)
+                                  (bytestructure-ref bs 'a)))
+      (test-eqv "set2" 987 (begin (bytestructure-set! bs 'b 987)
+                                  (bytestructure-ref bs 'b))))))
 
 (test-group "union"
   (test-assert "create" (bs:union `((x ,uint8) (y ,uint16))))
@@ -313,37 +358,47 @@
       (test-group "procedural"
         (define bs (bytestructure (bs:pointer uint16)))
         (define bv1 (make-bytevector 2))
-        (define address (ffi:pointer-address (ffi:bytevector->pointer bv1)))
-        (bytevector-address-set! (bytestructure-bytevector bs) 0 address)
+        (define bv2 (make-bytevector 4))
+        (define address1 (ffi:pointer-address (ffi:bytevector->pointer bv1)))
+        (define address2 (ffi:pointer-address (ffi:bytevector->pointer bv2)))
+        (bytevector-address-set! (bytestructure-bytevector bs) 0 address1)
         (bytevector-u16-native-set! bv1 0 321)
-        (test-eqv "ref" 321 (bytestructure-ref bs '*))
-        (test-eqv "set" 456 (begin (bytestructure-set! bs '* 456)
-                                   (bytestructure-ref bs '*)))
-        (test-eqv "ref2" address (bytestructure-ref bs))
+        (test-eqv "ref1" 321 (bytestructure-ref bs '*))
+        (test-eqv "set1" 456 (begin (bytestructure-set! bs '* 456)
+                                    (bytestructure-ref bs '*)))
+        (test-eqv "ref2" address1 (bytestructure-ref bs))
+        (test-eqv "set2" address2 (begin (bytestructure-set! bs address2)
+                                         (bytestructure-ref bs)))
+        (bytevector-address-set! (bytestructure-bytevector bs) 0 address2)
+        (bytevector-u16-native-set! bv2 2 456)
+        (test-eqv "ref3" 456 (bytestructure-ref bs 1))
+        (test-eqv "set3" 789 (begin (bytestructure-set! bs 1 789)
+                                    (bytestructure-ref bs 1)))
         (protect-from-gc-upto-here bv1)
-        (let* ((bv2 (make-bytevector 2 123))
-               (address (ffi:pointer-address (ffi:bytevector->pointer bv2))))
-          (test-eqv "set2" address (begin (bytestructure-set! bs address)
-                                          (bytestructure-ref bs)))
-          (protect-from-gc-upto-here bv2)))
+        (protect-from-gc-upto-here bv2))
       (test-group "syntactic"
         (define-bytestructure-accessors (bs:pointer uint16)
           unwrapper getter setter)
         (define bv (make-bytevector pointer-size))
         (define bv1 (make-bytevector 2))
-        (define address (ffi:pointer-address (ffi:bytevector->pointer bv1)))
-        (bytevector-address-set! bv 0 address)
+        (define bv2 (make-bytevector 4))
+        (define address1 (ffi:pointer-address (ffi:bytevector->pointer bv1)))
+        (define address2 (ffi:pointer-address (ffi:bytevector->pointer bv2)))
+        (bytevector-address-set! bv 0 address1)
         (bytevector-u16-native-set! bv1 0 321)
         (test-eqv "ref" 321 (getter bv *))
         (test-eqv "set" 456 (begin (setter bv * 456)
                                    (getter bv *)))
-        (test-eqv "ref2" address (getter bv))
+        (test-eqv "ref2" address1 (getter bv))
+        (test-eqv "set2" address1 (begin (setter bv address1)
+                                         (getter bv)))
+        (bytevector-address-set! bv 0 address2)
+        (bytevector-u16-native-set! bv2 2 456)
+        (test-eqv "ref3" 456 (getter bv 1))
+        (test-eqv "set3" 789 (begin (setter bv 1 789)
+                                    (getter bv 1)))
         (protect-from-gc-upto-here bv1)
-        (let* ((bv2 (make-bytevector 2 123))
-               (address (ffi:pointer-address (ffi:bytevector->pointer bv2))))
-          (test-eqv "set2" address (begin (setter bv address)
-                                          (getter bv)))
-          (protect-from-gc-upto-here bv2))))
+        (protect-from-gc-upto-here bv2)))
 
     (test-group "cstring-pointer"
       (let* ((cstr1-ptr (ffi:string->pointer "abc"))
@@ -368,8 +423,16 @@
  (else
   ))
 
+;; Do this before test-end since it removes the auto-inserted test runner.
+(define success
+  (let ((runner (test-runner-current)))
+    (and (zero? (test-runner-xpass-count runner))
+         (zero? (test-runner-fail-count runner)))))
+
 (test-end "bytestructures")
 
+(exit (if success 0 1))
+
 ;; Local Variables:
 ;; eval: (put (quote test-group) (quote scheme-indent-function) 1)
 ;; eval: (put (quote test-=) (quote scheme-indent-function) 2)
